SystemOrganization addCategory: #'CL-Benchmarks-Game'!
SystemOrganization addCategory: #'CL-Benchmarks-Game-Tests'!
SystemOrganization addCategory: #'CL-Benchmarks-Game-ChameneosRedux'!

TestCase subclass: #BGTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CL-Benchmarks-Game-Tests'!

!BGTests methodsFor: 'as yet unclassified' stamp: 'StefanMarr 5/15/2011 14:06'!
testBinaryTrees
	| bench output expected |
	bench := BGBinaryTrees new.
	output := (TextStream with: String new).

	self assert: -1 equals: (bench binarytrees: 10 to: output).
	
	expected := 'stretch tree of depth 11	 check: -1
2048	 trees of depth 4	 check: -2048
512	 trees of depth 6	 check: -512
128	 trees of depth 8	 check: -128
32	 trees of depth 10	 check: -32
long lived tree of depth 10	 check: -1
'.
	self assert: expected equals: output contents.! !

!BGTests methodsFor: 'as yet unclassified' stamp: 'StefanMarr 5/15/2011 14:08'!
testBinaryTreesResult
	| bench output |
	bench := BGBinaryTrees new.
				
	output := (TextStream with: String new).
	self assert: -1 equals: (bench binarytrees: 9 to: output).
	
	output := (TextStream with: String new).
	self assert: -1 equals: (bench binarytrees: 8 to: output).
	
	output := (TextStream with: String new).
	self assert: -1 equals: (bench binarytrees: 7 to: output).
	
	! !

!BGTests methodsFor: 'as yet unclassified' stamp: 'StefanMarr 5/15/2011 18:03'!
testChameleons
	self assert: (BGChameleons chameneos:     100) equals: 200. 
	self assert: (BGChameleons chameneos:   10000) equals: 20000.
	self assert: (BGChameleons chameneos: 1000000) equals: 2000000.! !

!BGTests methodsFor: 'as yet unclassified' stamp: 'StefanMarr 5/15/2011 18:03'!
testChameneosRedux
	| report result |
	
	report := TextStream with: String new.
	result := BGMall runBenchMark: 600 on: report.
	self assert: result equals: 1200.
	
	report := TextStream with: String new.
	result := BGMall runBenchMark: 555 on: report.
	self assert: result equals: (555 * 2).
	
	report := TextStream with: String new.
	result := BGMall runBenchMark: 121 on: report.
	self assert: result equals: (121 * 2).
	! !

!BGTests methodsFor: 'as yet unclassified' stamp: 'StefanMarr 5/15/2011 12:15'!
testFannkuchReduxResult
	 | f |
	f := BGFannkuchRedux fannkuchRedux: 7 to: (TextStream with: String new).
	self assert: f equals: 16! !

!BGTests methodsFor: 'as yet unclassified' stamp: 'StefanMarr 5/15/2011 18:07'!
testFasta
	| result |
	self flag: 'TODO: add proper verification'.
	self shouldnt: [result := BGFasta fasta] raise: Error.
	self assert: (result size > 0).
	self assert: (result isKindOf: String).
	! !

!BGTests methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/1/2012 22:33'!
testNBody
	| result |
	result := BGNBody nbody: 1000.
	
	self assert: (((result at: 2) asString) beginsWith: '-0.169087605')! !

SMarkSuite subclass: #BenchmarkGameSuite
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CL-Benchmarks-Game'!

!BenchmarkGameSuite class methodsFor: 'transporter' stamp: 'sm 5/25/2011 15:36'!
transportersForFileOutMenu
	^ { Transporter forPackage: (PackageInfo named: 'CL-Benchmarks-Game') }! !

!BenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 12/31/2011 16:14'!
benchBinaryTrees
	| bench problemSize |
	bench := BGBinaryTrees new.
	problemSize := self problemSize ifNotNil: [ self problemSize ]
							  ifNil: [ BGBinaryTrees defaultProblemSize ].
	^ bench binarytrees: problemSize to: (TextStream with: String new).! !

!BenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 12/31/2011 16:14'!
benchChameleons
	| result problemSize |
	problemSize := self problemSize ifNotNil: [ self problemSize ]
								  ifNil: [ BGChameleons defaultProblemSize ].
	result := BGChameleons chameneos: problemSize.
	
	"Result should be 2*numberOfProcesses (as far as I understand the benchmark)"
	self flag: 'TODO adapt once verification is supported'.
	result = (2 * problemSize) ifFalse: [
		Error signal: (BGChameleons name, ' benchmark failed. Input: ', 
								 problemSize asString, ' Output: ', result asString).
	].! !

!BenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 12/31/2011 16:14'!
benchChameneosRedux
	| result report problemSize |
	problemSize := self problemSize ifNotNil: [ self problemSize ]
								  ifNil: [ BGChameneosRedux defaultProblemSize ].
	report := TextStream with: String new.
	result := BGMall runBenchMark: problemSize on: report.
	
	"Result should be 2*problemSize"
	self flag: 'TODO: adapt once verification is supported'.
	result = (2 * problemSize) ifFalse: [
		Error signal: self class name, ' benchmark failed. Input: ',
								 runner processes asString, ' Output: ', result asString  ].! !

!BenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 1/10/2012 11:01'!
benchFannkuchRedux
   | f problemSize |
	problemSize := self problemSize ifNotNil: [ self problemSize ]
								  ifNil: [ BGFannkuchRedux defaultProblemSize ].
   f := BGFannkuchRedux fannkuchRedux: problemSize to: (TextStream with: String new).
   
	" the output should only be done by some reporter"
	"ScriptConsole print: 'Pfannkuchen(', problemSize asString , ') = ', f asString."
   ^f! !

!BenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 12/31/2011 16:15'!
benchFasta
	| out problemSize |
	out := TextStream with: String new.
	problemSize := self problemSize ifNotNil: [ self problemSize ]
								  ifNil: [ BGFasta defaultProblemSize ].  
	BGFasta fasta: problemSize to: out.! !

!BenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 12/31/2011 16:15'!
benchNBody
	| problemSize |
	problemSize := self problemSize ifNotNil: [ self problemSize ]
								  ifNil: [ BGNBody defaultProblemSize ].
	BGNBody nbody: problemSize.! !

!BenchmarkGameSuite methodsFor: 'license' stamp: 'StefanMarr 5/15/2011 12:31'!
license
	^
'Revised BSD license

This is a specific instance of the Open Source Initiative (OSI) BSD license
template http://www.opensource.org/licenses/bsd-license.php


Copyright 2008-2010 Isaac Gouy
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   Neither the name of "The Computer Language Benchmarks Game" nor the name of
   "The Computer Language Shootout Benchmarks" nor the name "nanobench" nor
   the name "bencher" nor the names of its contributors may be used to endorse
   or promote products derived from this software without specific prior
   written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.'! !

Object subclass: #BGBinaryTrees
	instanceVariableNames: 'left right item btFactory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CL-Benchmarks-Game'!

!BGBinaryTrees commentStamp: 'StefanMarr 1/16/2011 17:29' prior: 0!
This is an adaptation of a benchmark for testing GC. A whole tree is allocated before any nodes are GC'd.
The binary-trees benchmark is a simplistic adaptation of Hans Boehm's GCBench, which in turn was adapted from a benchmark by John Ellis and Pete Kovac.!

!BGBinaryTrees class methodsFor: 'defaults' stamp: 'StefanMarr 5/15/2011 23:25'!
defaultProblemSize
	^ 9! !

!BGBinaryTrees methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/31/2011 18:03'!
binaryTreeFactory: aBinaryTreeFactoryOrClass
	btFactory := aBinaryTreeFactoryOrClass! !

!BGBinaryTrees methodsFor: 'benchmarking' stamp: 'StefanMarr 12/31/2011 18:10'!
binarytrees: n to: output
   | minDepth maxDepth stretchDepth check longLivedTree iterations |
   minDepth := 4.
   maxDepth := minDepth + 2 max: n.
   stretchDepth := maxDepth + 1.

   check := (btFactory bottomUpTree: 0 depth: stretchDepth) itemCheck.
   output
      nextPutAll: 'stretch tree of depth '; print: stretchDepth; tab;
      nextPutAll: ' check: '; print: check; cr.

   longLivedTree := btFactory bottomUpTree: 0 depth: maxDepth
.
	self buildTreesFrom: minDepth to: maxDepth reportTo: output.   

	check := longLivedTree itemCheck.

   output
      nextPutAll: 'long lived tree of depth '; print: maxDepth; tab;
      nextPutAll: ' check: '; print: check; cr.
	
	^ check.! !

!BGBinaryTrees methodsFor: 'benchmarking' stamp: 'StefanMarr 12/31/2011 18:09'!
buildTreesFrom: minDepth to: maxDepth reportTo: output
	| iterations check |
   minDepth to: maxDepth by: 2 do: [:depth|
      iterations := 1 bitShift: maxDepth - depth + minDepth.

      check := 0.
      1 to: iterations do: [:i|
         check := check + (btFactory bottomUpTree: i depth: depth) itemCheck.
         check := check + (btFactory bottomUpTree: -1*i depth: depth) itemCheck
         ].
      output
         print:  (2*iterations); tab;
         nextPutAll: ' trees of depth '; print: depth; tab;
         nextPutAll: ' check: '; print: check; cr
      ].! !

!BGBinaryTrees methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/31/2011 18:06'!
initialize
	super initialize.
	"Set the default binaryTree class used for the benchmark"
	btFactory := BGTreeNode.! !

Object subclass: #BGBody
	instanceVariableNames: 'x y z vx vy vz mass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CL-Benchmarks-Game'!

!BGBody class methodsFor: 'constants' stamp: ''!
daysPerYear
   ^365.24d0! !

!BGBody class methodsFor: 'constants' stamp: ''!
jupiter
   ^self new
      x: 4.84143144246472090d0
      y: -1.16032004402742839d0
      z: -1.03622044471123109d-1
      vx: 1.66007664274403694d-3 * self daysPerYear
      vy: 7.69901118419740425d-3 * self daysPerYear
      vz: -6.90460016972063023d-5 * self daysPerYear
      mass: 9.54791938424326609d-4 * self solarMass! !

!BGBody class methodsFor: 'constants' stamp: ''!
neptune
   ^self new
      x: 1.53796971148509165d1
      y: -2.59193146099879641d1
      z: 1.79258772950371181d-1
      vx: 2.68067772490389322d-3 * self daysPerYear
      vy: 1.62824170038242295d-3 * self daysPerYear
      vz: -9.51592254519715870d-5 * self daysPerYear
      mass: 5.15138902046611451d-5 * self solarMass! !

!BGBody class methodsFor: 'constants' stamp: ''!
pi
   ^3.141592653589793d0! !

!BGBody class methodsFor: 'constants' stamp: ''!
saturn
   ^self new
      x: 8.34336671824457987d0
      y: 4.12479856412430479d0
      z: -4.03523417114321381d-1
      vx: -2.76742510726862411d-3 * self daysPerYear
      vy: 4.99852801234917238d-3 * self daysPerYear
      vz: 2.30417297573763929d-5 * self daysPerYear
      mass: 2.85885980666130812d-4 * self solarMass! !

!BGBody class methodsFor: 'constants' stamp: ''!
solarMass
   ^4.0d0 * self pi * self pi! !

!BGBody class methodsFor: 'constants' stamp: ''!
sun
   ^self new
      x: 0.0d0
      y: 0.0d0
      z: 0.0d0
      vx: 0.0d0
      vy: 0.0d0
      vz: 0.0d0
      mass: self solarMass! !

!BGBody class methodsFor: 'constants' stamp: ''!
uranus
   ^self new
      x: 1.28943695621391310d1
      y: -1.51111514016986312d1
      z: -2.23307578892655734d-1
      vx: 2.96460137564761618d-3 * self daysPerYear
      vy: 2.37847173959480950d-3 * self daysPerYear
      vz: -2.96589568540237556d-5 * self daysPerYear
      mass: 4.36624404335156298d-5 * self solarMass! !

!BGBody methodsFor: 'nbody' stamp: ''!
addMomentumTo: anArray
   anArray at: 1 put: (anArray at: 1) + (vx * mass).
   anArray at: 2 put: (anArray at: 2) + (vy * mass).
   anArray at: 3 put: (anArray at: 3) + (vz * mass).
   ^anArray! !

!BGBody methodsFor: 'nbody' stamp: ''!
and: aBody velocityAfter: dt        
   | dx dy dz distance mag |
   dx := x - aBody x.
   dy := y - aBody y.
   dz := z - aBody z.
   
   distance := ((dx*dx) + (dy*dy) + (dz*dz)) sqrt.
   mag := dt / (distance * distance * distance).

   self decreaseVelocity: dx y: dy z: dz m: aBody mass * mag.   
   aBody increaseVelocity: dx y: dy z: dz m: mass * mag! !

!BGBody methodsFor: 'nbody' stamp: ''!
decreaseVelocity: dx y: dy z: dz m: m
   vx := vx - (dx * m).
   vy := vy - (dy * m).
   vz := vz - (dz * m)! !

!BGBody methodsFor: 'nbody' stamp: ''!
increaseVelocity: dx y: dy z: dz m: m
   vx := vx + (dx * m).
   vy := vy + (dy * m).
   vz := vz + (dz * m)! !

!BGBody methodsFor: 'nbody' stamp: ''!
kineticEnergy
   ^0.5d0 * mass * ((vx * vx) + (vy * vy) + (vz * vz))! !

!BGBody methodsFor: 'accessing' stamp: ''!
mass
   ^mass! !

!BGBody methodsFor: 'nbody' stamp: ''!
offsetMomentum: anArray 
   | m |
   m := self class solarMass.
   vx := (anArray at: 1) negated / m.
   vy := (anArray at: 2) negated / m.
   vz := (anArray at: 3) negated / m! !

!BGBody methodsFor: 'nbody' stamp: ''!
positionAfter: dt
   x := x + (dt * vx).
   y := y + (dt * vy).
   z := z + (dt * vz)! !

!BGBody methodsFor: 'nbody' stamp: ''!
potentialEnergy: aBody
   | dx dy dz distance |
   dx := x - aBody x.
   dy := y - aBody y.
   dz := z - aBody z.

   distance := ((dx*dx) + (dy*dy) + (dz*dz)) sqrt.
   ^mass * aBody mass / distance! !

!BGBody methodsFor: 'accessing' stamp: ''!
x
   ^x! !

!BGBody methodsFor: 'accessing' stamp: ''!
x: d1 y: d2 z: d3 vx: d4 vy: d5 vz: d6 mass: d7
   x := d1.
   y := d2. 
   z := d3. 
   vx := d4.
   vy := d5.
   vz := d6.
   mass := d7! !

!BGBody methodsFor: 'accessing' stamp: ''!
y
   ^y! !

!BGBody methodsFor: 'accessing' stamp: ''!
z
   ^z! !

Object subclass: #BGChameleon
	instanceVariableNames: 'meetings color semaphore waitingForPair'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CL-Benchmarks-Game'!

!BGChameleon class methodsFor: 'instance creation' stamp: ''!
color: c
    ^self new
        initialize;
        color: c! !

!BGChameleon methodsFor: 'accessing' stamp: ''!
color
    ^color! !

!BGChameleon methodsFor: 'changing colours' stamp: ''!
color: c
    color := c! !

!BGChameleon methodsFor: 'changing colours' stamp: ''!
fade
    color := #faded.
    waitingForPair signal.
    semaphore signal! !

!BGChameleon methodsFor: 'running' stamp: ''!
fork: meetingPlace
    ^[ self run: meetingPlace ] fork! !

!BGChameleon methodsFor: 'initialize-release' stamp: ''!
initialize
    meetings := 0.
    waitingForPair := Semaphore new.
    semaphore := Semaphore new! !

!BGChameleon methodsFor: 'accessing' stamp: ''!
meetings
    ^meetings! !

!BGChameleon methodsFor: 'changing colours' stamp: ''!
met: other
    | newColor |
    meetings := meetings + 1.
    color == #red ifTrue: [
    newColor := other == #yellow ifTrue: [ #blue ] ifFalse: [ #yellow ] ].
    color == #yellow ifTrue: [
    newColor := other == #red ifTrue: [ #blue ] ifFalse: [ #red ] ].
    color == #blue ifTrue: [
    newColor := other == #red ifTrue: [ #yellow ] ifFalse: [ #red ] ].
    color := newColor.
    waitingForPair signal! !

!BGChameleon methodsFor: 'running' stamp: ''!
run: meetingPlace
    [ color == #faded ] whileFalse: [
        meetingPlace reachedBy: self.
        waitingForPair wait ]! !

!BGChameleon methodsFor: 'accessing' stamp: ''!
wait
    semaphore wait! !

Object subclass: #BGChameleons
	instanceVariableNames: 'meetings color semaphore waitingForPair'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CL-Benchmarks-Game'!

!BGChameleons class methodsFor: 'benchmark scripts' stamp: 'StefanMarr 1/16/2011 12:12'!
chameneos
	" reports back the number of meetings "
	| out |
	^ self chameneos: 1000000! !

!BGChameleons class methodsFor: 'benchmarking' stamp: 'StefanMarr 5/15/2011 12:17'!
chameneos: n
    | c1 c2 c3 c4 mp |
    c1 := BGChameleon color: #blue.
    c2 := BGChameleon color: #red.
    c3 := BGChameleon color: #yellow.
    c4 := BGChameleon color: #blue.
    mp := BGMeetingPlace forMeetings: n.
    c1 fork: mp.
    c2 fork: mp.
    c3 fork: mp.
    c4 fork: mp.
    c1 wait.
    c2 wait.
    c3 wait.
    c4 wait.
    ^c1 meetings + c2 meetings + c3 meetings + c4 meetings! !

!BGChameleons class methodsFor: 'defaults' stamp: 'StefanMarr 5/15/2011 23:31'!
defaultProblemSize
	^ 10000! !

Object subclass: #BGChameneosColour
	instanceVariableNames: 'color'
	classVariableNames: 'Blue Red Yellow'
	poolDictionaries: ''
	category: 'CL-Benchmarks-Game-ChameneosRedux'!

!BGChameneosColour class methodsFor: 'accessing' stamp: ''!
blue
   ^Blue! !

!BGChameneosColour class methodsFor: 'accessing' stamp: ''!
blue: anObject
   Blue := anObject! !

!BGChameneosColour class methodsFor: 'initialize-release' stamp: ''!
createBlue
   "comment stating purpose of message"

   ^super new color: #blue! !

!BGChameneosColour class methodsFor: 'initialize-release' stamp: ''!
createRed
   "comment stating purpose of message"

   ^super new color: #red! !

!BGChameneosColour class methodsFor: 'initialize-release' stamp: ''!
createYellow
   "comment stating purpose of message"

   ^super new color: #yellow! !

!BGChameneosColour class methodsFor: 'printing' stamp: 'StefanMarr 1/16/2011 14:13'!
generateReportOfColoursOn: readOut 
   | colours |
   colours := Array 
            with: Blue
            with: Red
            with: Yellow.
   colours do: 
         [:aColour | 
         colours do: 
               [:anotherColour | 
               aColour printOn: readOut.
               readOut nextPutAll: ' + '.
               anotherColour printOn: readOut.
               readOut nextPutAll: ' -> '.
               (aColour complementaryColourFor: anotherColour) printOn: readOut.
               readOut cr]].
   ^readOut! !

!BGChameneosColour class methodsFor: 'initialize-release' stamp: ''!
initialize
   "self initialize"

   Red := self createRed.
   Blue := self createBlue.
   Yellow := self createYellow! !

!BGChameneosColour class methodsFor: 'accessing' stamp: ''!
red
   ^Red! !

!BGChameneosColour class methodsFor: 'accessing' stamp: ''!
red: anObject
   Red := anObject! !

!BGChameneosColour class methodsFor: 'accessing' stamp: ''!
yellow
   ^Yellow! !

!BGChameneosColour class methodsFor: 'accessing' stamp: ''!
yellow: anObject
   Yellow := anObject! !

!BGChameneosColour methodsFor: 'accessing' stamp: ''!
color
   ^color! !

!BGChameneosColour methodsFor: 'accessing' stamp: ''!
color: aColor 
   color := aColor! !

!BGChameneosColour methodsFor: 'as yet unclassified' stamp: ''!
complementaryColourFor: aChameneosColour 
   "determine the complementary colour defined as..."

   self == aChameneosColour ifTrue: [^self].
   self isBlue 
      ifTrue: 
         [aChameneosColour isRed 
            ifTrue: [^self class yellow]
            ifFalse: [^self class red]].
   self isRed 
      ifTrue: 
         [aChameneosColour isBlue 
            ifTrue: [^self class yellow]
            ifFalse: [^self class blue]].
   aChameneosColour isBlue 
      ifTrue: [^self class red]
      ifFalse: [^self class blue]! !

!BGChameneosColour methodsFor: 'testing' stamp: ''!
hasSameColorAs: aChameneos 
   ^self color == aChameneos color! !

!BGChameneosColour methodsFor: 'testing' stamp: ''!
isBlue
   ^self == self class blue! !

!BGChameneosColour methodsFor: 'testing' stamp: ''!
isRed
   ^self == self class red! !

!BGChameneosColour methodsFor: 'testing' stamp: ''!
isYellow
   ^self == self class yellow! !

!BGChameneosColour methodsFor: 'printing' stamp: ''!
printOn: aStream 
   aStream nextPutAll: self color! !

Object subclass: #BGChameneosRedux
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CL-Benchmarks-Game-ChameneosRedux'!

!BGChameneosRedux class methodsFor: 'benchmarking-scripts' stamp: 'StefanMarr 5/15/2011 12:27'!
chameneosredux2
	"ChameneosRedux chameneosredux2" 
	| report |
	report := TextStream with: String new.
	BGMall runBenchMark: 600 on: report.
	^report contents! !

!BGChameneosRedux class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 5/15/2011 23:34'!
defaultProblemSize
	^ 6000! !

Object subclass: #BGCreature
	instanceVariableNames: 'creatureName colour selfMet creaturesMet'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CL-Benchmarks-Game-ChameneosRedux'!

!BGCreature class methodsFor: 'initialize-release' stamp: 'StefanMarr 5/15/2011 12:28'!
withName: aName colour: aColour 
   ^(BGCreature new initialize)
      name: aName;
      colour: aColour! !

!BGCreature methodsFor: 'accessing' stamp: ''!
colour
   ^colour! !

!BGCreature methodsFor: 'accessing' stamp: ''!
colour: anObject 
   colour := anObject! !

!BGCreature methodsFor: 'accessing' stamp: ''!
creaturesMet
   ^creaturesMet! !

!BGCreature methodsFor: 'accessing' stamp: ''!
creaturesMet: anObject 
   creaturesMet := anObject! !

!BGCreature methodsFor: 'initialize-release' stamp: ''!
initialize
   selfMet := 0.
   creaturesMet := 0! !

!BGCreature methodsFor: 'accessing' stamp: ''!
name
   ^creatureName! !

!BGCreature methodsFor: 'accessing' stamp: ''!
name: anObject 
   creatureName := anObject! !

!BGCreature methodsFor: 'accessing' stamp: ''!
selfMet
   ^selfMet! !

!BGCreature methodsFor: 'accessing' stamp: ''!
selfMet: anObject 
   ^selfMet := anObject! !

!BGCreature methodsFor: 'controlling' stamp: ''!
visitMall: mall 
   
   [| partner |
   partner := mall visitWith: self.
   partner ifNotNil: 
         [colour := colour complementaryColourFor: partner colour.
         self == partner ifTrue: [selfMet := selfMet + 1].
         creaturesMet := creaturesMet + 1].
   partner isNil] 
         whileFalse! !

Object subclass: #BGFannkuchRedux
	instanceVariableNames: 'timesRotated perm atEnd'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CL-Benchmarks-Game'!

!BGFannkuchRedux class methodsFor: 'defaults' stamp: 'StefanMarr 5/15/2011 23:35'!
defaultProblemSize
	^ 4! !

!BGFannkuchRedux class methodsFor: 'benchmarking' stamp: 'StefanMarr 1/10/2012 10:54'!
fannkuchRedux: n to: output
	| gen |
	gen := BGPermGeneratorRedux new: n.
   ^ self maxPfannkuchenTo: output with: gen.! !

!BGFannkuchRedux class methodsFor: 'benchmark scripts' stamp: 'StefanMarr 5/16/2011 14:45'!
fannkuchredux
   | n f |
   n := 4.
   f := self fannkuchRedux: n to: (TextStream with: String new).
   ScriptConsole println: 'Pfannkuchen(', n printString, ') = ', f asString.
   ^''! !

!BGFannkuchRedux class methodsFor: 'benchmarking' stamp: 'StefanMarr 1/10/2012 10:55'!
maxPfannkuchenTo: output with: gen
   | max permutation checksum permCount flipsCount |
   max := 0.
   permCount := 0.
   checksum := 0.
   [gen atEnd] whileFalse:
      [permutation := gen next.
      permCount := permCount + 1.
      (permCount = 1048576) ifTrue: [permCount := 0].
      flipsCount := self pfannkuchen: permutation.
      checksum := permCount odd ifTrue: [checksum+flipsCount] ifFalse: [checksum-flipsCount].
      max := max max: flipsCount].
   output print: checksum; cr.
   ^max! !

!BGFannkuchRedux class methodsFor: 'benchmarking' stamp: 'StefanMarr 1/10/2012 10:55'!
pfannkuchen: anArray
   | first complement a b k |
   k := 0.
   [ (first := anArray at: 1) == 1 ] whileFalse: [
      k := k + 1.
      complement := first + 1.
      1 to: first // 2 do: [ :i |
         a := anArray at: i.
         b := anArray at: complement - i.
         anArray at: i put: b.
         anArray at: complement - i put: a.
      ]
   ].
   ^k! !

Object subclass: #BGFasta
	instanceVariableNames: 'lineLength n out r repeatStreamFact randomStreamFact'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CL-Benchmarks-Game'!

!BGFasta class methodsFor: 'benchmark scripts' stamp: 'StefanMarr 5/15/2011 23:20'!
defaultProblemSize
	^ 1000! !

!BGFasta class methodsFor: 'benchmark scripts' stamp: 'StefanMarr 1/2/2012 10:44'!
fasta
	" self fasta"
  | out |
  out := TextStream with: String new.
  self fasta: self defaultProblemSize to: out.
  ^out contents! !

!BGFasta class methodsFor: 'benchmarking' stamp: 'StefanMarr 1/2/2012 10:27'!
fasta: n to: out
   | r lineLength bench |

	bench := self new.
	bench n: n.
	bench out: out.
	bench oneHomoSapiensAlu.
	
   bench r: (BGRandomNumber to: 1). "Shared random sequence"

	bench twoIUBAmbiguityCodes.
	bench threeHomoSapiensFrequency.
	
	out flush.! !

!BGFasta methodsFor: 'initialization' stamp: 'StefanMarr 1/2/2012 10:33'!
initialize
	super initialize.
	lineLength := 60.
	repeatStreamFact := BGRepeatStream.
	randomStreamFact := BGRandomStream.! !

!BGFasta methodsFor: 'accessing' stamp: 'StefanMarr 1/2/2012 10:21'!
n: anInteger
	n := anInteger! !

!BGFasta methodsFor: 'benchmarking' stamp: 'StefanMarr 1/4/2012 11:09'!
oneHomoSapiensAlu
	 self
      writeFasta: 'ONE Homo sapiens alu'
      from:
         ( repeatStreamFact
            to: n*2
            on:'GGCCGGGCGCGGTGGCTCACGCCTGTAATCCCAGCACTTTGG',
               'GAGGCCGAGGCGGGCGGATCACCTGAGGTCAGGAGTTCGAGA',
               'CCAGCCTGGCCAACATGGTGAAACCCCGTCTCTACTAAAAAT',
               'ACAAAAATTAGCCGGGCGTGGTGGCGCGCGCCTGTAATCCCA',
               'GCTACTCGGGAGGCTGAGGCAGGAGAATCGCTTGAACCCGGG',
               'AGGCGGAGGTTGCAGTGAGCCGAGATCGCGCCACTGCACTCC',
               'AGCCTGGGCGACAGAGCGAGACTCCGTCTCAAAAA' ).! !

!BGFasta methodsFor: 'accessing' stamp: 'StefanMarr 1/2/2012 10:21'!
out: aStream
	out := aStream ! !

!BGFasta methodsFor: 'accessing' stamp: 'StefanMarr 1/2/2012 10:25'!
r: aRandomNumberStream
	r := aRandomNumberStream! !

!BGFasta methodsFor: 'accessing' stamp: 'StefanMarr 1/2/2012 10:34'!
randomStreamFact: aStreamFact
	randomStreamFact := aStreamFact! !

!BGFasta methodsFor: 'accessing' stamp: 'StefanMarr 1/2/2012 10:34'!
repeatStreamFact: aStreamFact
	repeatStreamFact := aStreamFact! !

!BGFasta methodsFor: 'benchmarking' stamp: 'StefanMarr 1/4/2012 11:10'!
threeHomoSapiensFrequency
	self
      writeFasta: 'THREE Homo sapiens frequency'
      from:
         (( randomStreamFact
            to: n*5
            on: #(   #($a 0.3029549426680d0)
                  #($c 0.1979883004921d0)
                  #($g 0.1975473066391d0)
                  #($t 0.3015094502008d0)))
            random: r;
            yourself).! !

!BGFasta methodsFor: 'benchmarking' stamp: 'StefanMarr 1/4/2012 15:03'!
twoIUBAmbiguityCodes
	self
      writeFasta: 'TWO IUB ambiguity codes'
      from:
         (( randomStreamFact
            to: n*3
            on: #(   #($a 0.27d0)
                  #($c 0.12d0)
                  #($g 0.12d0)
                  #($t 0.27d0)

                  #($B 0.02d0)
                  #($D 0.02d0)
                  #($H 0.02d0)
                  #($K 0.02d0)
                  #($M 0.02d0)
                  #($N 0.02d0)
                  #($R 0.02d0)
                  #($S 0.02d0)
                  #($V 0.02d0)
                  #($W 0.02d0)
                  #($Y 0.02d0))
		) random: r;
         	yourself).! !

!BGFasta methodsFor: 'benchmarking' stamp: 'StefanMarr 1/4/2012 11:09'!
writeFasta: aString from: inStream
   | i |
   out nextPut: $>; nextPutAll: aString; cr.
   i := 0.
   [inStream atEnd] whileFalse:
      [i == lineLength ifTrue: [out cr. i := 0].
      out nextPut: inStream next.
      i := i + 1].
   out cr! !

Object subclass: #BGMall
	instanceVariableNames: 'guard maxRendezvous open process queue cache pairCache'
	classVariableNames: 'Units'
	poolDictionaries: ''
	category: 'CL-Benchmarks-Game-ChameneosRedux'!

BGMall class
	instanceVariableNames: 'lastSum'!

BGMall class
	instanceVariableNames: 'lastSum'!

!BGMall class methodsFor: 'initialize-release' stamp: ''!
createAllowing: maxRendezvous 
   "Private"

   ^self basicNew initialize maxRendezvous: maxRendezvous! !

!BGMall class methodsFor: 'initialize-release' stamp: 'StefanMarr 5/15/2011 12:29'!
createCreaturesWith: aCollectionOfColours 
   "Private"

   | aName |
   aName := 0.
   ^aCollectionOfColours collect: 
         [:aColour | 
         aName := aName + 1.
         BGCreature withName: aName colour: aColour]! !

!BGMall class methodsFor: 'printing' stamp: 'StefanMarr 1/16/2011 14:42'!
generateReportFor: creatures printOn: stream 
   | sum |
   sum := creatures inject: 0 into: [:accum :each | accum + each creaturesMet].
   creatures do: 
         [:aCreature | 
         aCreature creaturesMet printOn: stream.
         stream
            space;
            nextPutAll: (self units at: aCreature selfMet + 1);
            cr].
   stream space.
   sum printString 
      do: [:el | stream nextPutAll: (self units at: el digitValue + 1)]
      separatedBy: [stream space].
	lastSum := sum.
   ^stream! !

!BGMall class methodsFor: 'printing' stamp: ''!
generateReportForColours: colours printOn: stream 
   stream space.
   colours do: [:colour | colour printOn: stream] separatedBy: [stream space].
   ^stream! !

!BGMall class methodsFor: 'initialize-release' stamp: ''!
initialize
   "self initialize"

   Units := #('zero' 'one' 'two' 'three' 'four' 'five' 'six' 'seven' 'eight' 'nine')! !

!BGMall class methodsFor: 'initialize-release' stamp: ''!
new
   ^self shouldNotImplement! !

!BGMall class methodsFor: 'private' stamp: ''!
openMall: aMall forCreatures: creatures usingGuard: sema 
   | processes |
   processes := creatures 
            collect: [:aCreature | 
               [aCreature visitMall: aMall.
               sema signal] newProcess].
   processes do: 
         [:proc | 
         proc priority: Processor userBackgroundPriority.
         proc resume]! !

!BGMall class methodsFor: 'initialize-release' stamp: ''!
openMallWith: aCollectionOfColours forNumberOfMeets: aNumber 
   | mall creatures guard |
   mall := self createAllowing: aNumber.
   mall run.
   creatures := self createCreaturesWith: aCollectionOfColours.
   guard := Semaphore new.
   self 
      openMall: mall
      forCreatures: creatures
      usingGuard: guard.
   self 
      waitForClosingOfMall: mall
      withCreatures: creatures
      usingGuard: guard.
   ^creatures! !

!BGMall class methodsFor: 'public' stamp: 'StefanMarr 5/15/2011 12:28'!
runBenchMark: number on: anOutputStream 
   "self runBenchMark: 60000 on: Transcript."

   | firstTestColours secondTestColours blue red yellow creatures |
   blue := BGChameneosColour blue.
   red := BGChameneosColour red.
   yellow := BGChameneosColour yellow.
   firstTestColours := Array 
            with: blue
            with: red
            with: yellow.
   secondTestColours := (OrderedCollection new)
            add: blue;
            add: red;
            add: yellow;
            add: red;
            add: yellow;
            add: blue;
            add: red;
            add: yellow;
            add: red;
            add: blue;
            yourself.
   (BGChameneosColour generateReportOfColoursOn: anOutputStream) cr.
   (self generateReportForColours: firstTestColours printOn: anOutputStream) 
      cr.
   creatures := BGMall openMallWith: firstTestColours forNumberOfMeets: number.
   (self generateReportFor: creatures printOn: anOutputStream)
      cr;
      cr.
   (self generateReportForColours: secondTestColours printOn: anOutputStream) 
      cr.
   creatures := BGMall openMallWith: secondTestColours forNumberOfMeets: number.
   (self generateReportFor: creatures printOn: anOutputStream)
      cr;
      cr.
	^ lastSum! !

!BGMall class methodsFor: 'accessing' stamp: ''!
units
   ^Units! !

!BGMall class methodsFor: 'private' stamp: ''!
waitForClosingOfMall: aMall withCreatures: creatures usingGuard: guard 
   creatures size timesRepeat: [guard wait].
   aMall close! !

!BGMall methodsFor: 'controlling' stamp: ''!
close
   open := false! !

!BGMall methodsFor: 'initialize-release' stamp: 'StefanMarr 5/15/2011 12:27'!
initialize
   guard := Semaphore forMutualExclusion.
   queue := SharedQueue new.
   cache := OrderedCollection new.
   1 to: 10 do: [:x | cache add: BGPair new]! !

!BGMall methodsFor: 'accessing' stamp: ''!
maxRendezvous: max 
   maxRendezvous := max! !

!BGMall methodsFor: 'private' stamp: ''!
obtainPair
   ^cache removeFirst! !

!BGMall methodsFor: 'private' stamp: ''!
processVisitors
   [open] whileTrue: 
         [1 to: maxRendezvous
            do: 
               [:x | 
               | first second |
               first := queue next.
               second := queue next.
               self setPartnersOn: first and: second.
               first signal.
               second signal].
         [queue isEmpty] whileFalse: [queue next signal]].
   process terminate.
   process := nil! !

!BGMall methodsFor: 'private' stamp: ''!
releasePair: pair 
   pair release.
   cache addFirst: pair! !

!BGMall methodsFor: 'initialize-release' stamp: ''!
run
   open := true.
   process ifNil: 
         [process := [self processVisitors] newProcess.
         process priority: Processor userBackgroundPriority -1 ].
   process resume! !

!BGMall methodsFor: 'private' stamp: ''!
setPartnersOn: first and: second
   first partner: second me.
   second partner: first me.
! !

!BGMall methodsFor: 'private' stamp: ''!
shutdown
   [queue isEmpty] whileFalse: [queue next signal].
   process terminate.
   process := nil! !

!BGMall methodsFor: 'controlling' stamp: ''!
visitWith: aChameneos 
   | pair partner |
   pair := self obtainPair.
   pair me: aChameneos.
   queue nextPut: pair.
   pair wait.
   partner := pair partner.
   self releasePair: pair.
   ^partner! !

Object subclass: #BGMeetingPlace
	instanceVariableNames: 'mutex first total max'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CL-Benchmarks-Game'!

!BGMeetingPlace class methodsFor: 'instance creation' stamp: ''!
forMeetings: maxMeetings
    ^super new
        initialize;
        max: maxMeetings;
        yourself! !

!BGMeetingPlace methodsFor: 'initialize-release' stamp: ''!
initialize
    mutex := Semaphore forMutualExclusion.
    total := 0! !

!BGMeetingPlace methodsFor: 'running' stamp: ''!
max: maxMeetings
    max := maxMeetings! !

!BGMeetingPlace methodsFor: 'running' stamp: ''!
organizeMeetingWith: second
    total >= max
        ifTrue: [
            first fade.
            second fade ]
        ifFalse: [
            first met: second color.
            second met: first color ].
    total := total + 1! !

!BGMeetingPlace methodsFor: 'running' stamp: ''!
reachedBy: chameleon
    mutex critical: [
        first isNil
            ifTrue: [ first := chameleon ]
            ifFalse: [ self organizeMeetingWith: chameleon. first := nil ] ]! !

Object subclass: #BGNBody
	instanceVariableNames: 'bodies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CL-Benchmarks-Game'!

!BGNBody class methodsFor: 'defaults' stamp: 'StefanMarr 5/15/2011 23:36'!
defaultProblemSize
	^ 1000! !

!BGNBody class methodsFor: 'benchmark scripts' stamp: 'StefanMarr 1/15/2012 18:57'!
nbody: times
	"self nbody: 1000"
	^ self nbody: times with: self new! !

!BGNBody class methodsFor: 'benchmark scripts' stamp: 'StefanMarr 1/1/2012 22:36'!
nbody: times with: bodies
	"self nbody: 1000"
   | initEnergy finalEnergy |

	initEnergy := bodies energy.

   times timesRepeat: [bodies after: 0.01d0].
	
	finalEnergy := bodies energy.
	
   ^ {initEnergy. finalEnergy}! !

!BGNBody methodsFor: 'nbody' stamp: 'StefanMarr 1/15/2012 19:00'!
after: dt 
	1 to: bodies size do: [:i|
		i + 1 to: bodies size do: [:j|
			(bodies at: i) and: (bodies at: j) velocityAfter: dt]].

	bodies do: [:t6 | t6 positionAfter: dt]! !

!BGNBody methodsFor: 'nbody' stamp: 'StefanMarr 1/15/2012 19:02'!
energy
	| e |
	e := 0.0.
	1 to: bodies size do: [:i | 
		e := e + (bodies at: i) kineticEnergy.
		i + 1 to: bodies size do: [:j |
			e := e - ((bodies at: i) potentialEnergy: (bodies at: j))]
	].
	^ e! !

!BGNBody methodsFor: 'initialization' stamp: 'StefanMarr 1/15/2012 19:03'!
initialize
	bodies := OrderedCollection new add: BGBody sun;
				 add: BGBody jupiter;
				 add: BGBody saturn;
				 add: BGBody uranus;
				 add: BGBody neptune;
				 yourself.
	
	bodies first offsetMomentum: (bodies
				inject: (Array with: 0.0 with: 0.0 with: 0.0)
				into: [:m :each | each addMomentumTo: m])! !

Object subclass: #BGPair
	instanceVariableNames: 'partner me sema'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CL-Benchmarks-Game-ChameneosRedux'!

!BGPair class methodsFor: 'instance creation' stamp: ''!
new
   "Answer a newly created and initialized instance."
   ^super new initialize.! !

!BGPair class methodsFor: 'instance creation' stamp: ''!
with: me 
   "Answer a newly created and initialized instance."
self halt.
   ^super new initialize me: me! !

!BGPair methodsFor: 'initialize-release' stamp: ''!
initialize
   "Initialize a newly created instance. This method must answer the receiver."

   partner := nil.
   me := nil.
   sema := Semaphore new.
   ^self! !

!BGPair methodsFor: 'accessing' stamp: ''!
me
   ^me! !

!BGPair methodsFor: 'accessing' stamp: ''!
me: anObject
   me := anObject! !

!BGPair methodsFor: 'accessing' stamp: ''!
partner
   ^partner! !

!BGPair methodsFor: 'accessing' stamp: ''!
partner: anObject
   partner := anObject! !

!BGPair methodsFor: 'initialize-release' stamp: ''!
release
partner:=nil.! !

!BGPair methodsFor: 'initialize-release' stamp: ''!
signal
   sema signal! !

!BGPair methodsFor: 'initialize-release' stamp: ''!
wait
   sema wait! !

Object subclass: #BGPermGeneratorRedux
	instanceVariableNames: 'timesRotated perm atEnd'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CL-Benchmarks-Game'!

!BGPermGeneratorRedux class methodsFor: 'instance creation' stamp: ''!
new: size
   ^self new
      initialize: size;
      yourself! !

!BGPermGeneratorRedux methodsFor: 'accessing' stamp: ''!
atEnd
   ^atEnd! !

!BGPermGeneratorRedux methodsFor: 'initialize-release' stamp: 'StefanMarr 1/15/2012 20:36'!
initialize: size
   perm := (1 to: size) asArray.
   timesRotated := Array new: size withAll: 0.
   atEnd := false.! !

!BGPermGeneratorRedux methodsFor: 'initialize-release' stamp: ''!
makeNext
   | temp remainder |
   "* Generate the next permutation. *"
   2 to: perm size do: [ :r |
      "* Rotate the first r items to the left. *"
      temp := perm at: 1.
      1 to: r - 1 do: [ :i | perm at: i put: (perm at: i + 1) ].
      perm at: r put: temp.

      remainder := timesRotated at: r put: ((timesRotated at: r) + 1) \\ r.
      remainder = 0 ifFalse: [ ^self ].

      "* After r rotations, the first r items are in their original positions.
      Go on rotating the first r+1 items. *"
   ].

   "* We are past the final permutation. *"
   atEnd := true! !

!BGPermGeneratorRedux methodsFor: 'accessing' stamp: ''!
next
   | result |
   result := perm copy.
   self makeNext.
   ^result! !

Object subclass: #BGRandomNumber
	instanceVariableNames: 'seed scale'
	classVariableNames: 'FModulus Increment Modulus Multiplier'
	poolDictionaries: ''
	category: 'CL-Benchmarks-Game'!

!BGRandomNumber class methodsFor: 'class initialization' stamp: ''!
initialize
   FModulus := 139968.0d0.
   Increment := 29573.
   Modulus := 139968.
   Multiplier := 3877.! !

!BGRandomNumber class methodsFor: 'initialize-release' stamp: ''!
to: anInteger
   ^self basicNew to: anInteger! !

!BGRandomNumber methodsFor: 'accessing' stamp: ''!
next
   seed := (seed * Multiplier + Increment) \\ Modulus.
   ^(seed * scale) / FModulus! !

!BGRandomNumber methodsFor: 'private' stamp: ''!
to: anInteger
   seed := 42.
   scale := anInteger! !

Object subclass: #BGTreeNode
	instanceVariableNames: 'left right item'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CL-Benchmarks-Game'!

!BGTreeNode class methodsFor: 'instance creation' stamp: ''!
bottomUpTree: anItem depth: anInteger
   ^(anInteger > 0) 
      ifTrue: [
         self 
            left: (self bottomUpTree: 2*anItem - 1 depth: anInteger - 1) 
            right: (self bottomUpTree: 2*anItem depth: anInteger - 1)  
            item: anItem
         ]
      ifFalse: [self left: nil right: nil item: anItem]! !

!BGTreeNode class methodsFor: 'instance creation' stamp: ''!
left: leftChild right: rightChild item: anItem      
   ^(super new) left: leftChild right: rightChild item: anItem! !

!BGTreeNode methodsFor: 'accessing' stamp: ''!
itemCheck
   ^left isNil 
      ifTrue: [item] ifFalse: [item + (left itemCheck - right itemCheck)]! !

!BGTreeNode methodsFor: 'initialize-release' stamp: ''!
left: leftChild right: rightChild item: anItem
   left := leftChild.
   right := rightChild.
   item := anItem! !

ReadStream subclass: #BGRepeatStream
	instanceVariableNames: 'repeatPtr repeatLimit omniDomain'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CL-Benchmarks-Game'!

BGRepeatStream subclass: #BGRandomStream
	instanceVariableNames: 'random percentages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CL-Benchmarks-Game'!

!BGRandomStream methodsFor: 'accessing' stamp: ''!
next
   | r |
   r := random next.
   repeatPtr := repeatPtr + 1.
   1 to: percentages size do: [:i|
      (r < (percentages at: i)) ifTrue: [^collection at: i]]! !

!BGRandomStream methodsFor: 'initialize-release' stamp: 'StefanMarr 5/15/2011 12:19'!
on: aCollection
   | size cp |
   repeatPtr := 0.
   random := BGRandomNumber to: 1.0d0.
   size := aCollection size.
   percentages := Array new: size.
   collection := Array new: size.
   cp := 0.0d0.
   1 to: size do: [:i|
      collection at: i put: (aCollection at: i) first.
      percentages at: i put: (cp := cp + (aCollection at: i) last).
   ]! !

!BGRandomStream methodsFor: 'accessing' stamp: ''!
random: aRandomNumber
"* Share the random number generator so we can get the expected results. *"
   random := aRandomNumber! !

!BGRepeatStream class methodsFor: 'instance creation' stamp: ''!
to: anInteger on: aCollection
   ^(super on: aCollection) to: anInteger! !

!BGRepeatStream methodsFor: 'testing' stamp: ''!
atEnd
   ^repeatPtr >= repeatLimit! !

!BGRepeatStream methodsFor: 'accessing' stamp: ''!
next
   position >= readLimit ifTrue: [ self position: 0 ].
   repeatPtr := repeatPtr + 1.
   ^collection at: (position := position + 1)! !

!BGRepeatStream methodsFor: 'omni reflection' stamp: ''!
omniDomain
	^ omniDomain! !

!BGRepeatStream methodsFor: 'omni reflection' stamp: ''!
omniDomain: t1 
	omniDomain := t1! !

!BGRepeatStream methodsFor: 'omni reflection' stamp: ''!
requestExecutionOf: t1 
	<omniDoNotTransform>
	^ omniDomain requestExecutionOf: t1 on: self! !

!BGRepeatStream methodsFor: 'omni reflection' stamp: ''!
requestExecutionOf: t1 lookup: t2 
	<omniDoNotTransform>
	^ omniDomain
		requestExecutionOf: t1
		lookup: t2
		on: self! !

!BGRepeatStream methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:02'!
requestExecutionOf: aSel with: p1
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		of: aSel
		on: self! !

!BGRepeatStream methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:23'!
requestExecutionOf: aSel with: p1 with: p2
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		with: p2
		of: aSel
		on: self! !

!BGRepeatStream methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:31'!
requestExecutionOf: aSel with: p1 with: p2 with: p3 
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		with: p2
		with: p3
		of: aSel
		on: self! !

!BGRepeatStream methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:33'!
requestExecutionOf: aSel with: p1 with: p2 with: p3 with: p4
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		with: p2
		with: p3
		with: p4
		of: aSel
		on: self! !

!BGRepeatStream methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:36'!
requestExecutionOf: aSel with: p1 with: p2 with: p3 with: p4 with: p5
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		with: p2
		with: p3
		with: p4
		with: p5
		of: aSel
		on: self! !

!BGRepeatStream methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:37'!
requestExecutionOf: aSel with: p1 with: p2 with: p3 with: p4 with: p5 with: p6
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		with: p2
		with: p3
		with: p4
		with: p5
		with: p6
		of: aSel
		on: self! !

!BGRepeatStream methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:38'!
requestExecutionOf: aSel with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		with: p2
		with: p3
		with: p4
		with: p5
		with: p6
		with: p7
		of: aSel
		on: self! !

!BGRepeatStream methodsFor: 'omni reflection' stamp: ''!
requestExecutionOf: t1 with: t2 with: t3 with: t4 with: t5 with: t6 with: t7 with: t8 with: t9 
	<omniDoNotTransform>
	^ omniDomain
		requestExecutionOf: t1
		on: self
		with: t2
		with: t3
		with: t4
		with: t5
		with: t6
		with: t7
		with: t8
		with: t9! !

!BGRepeatStream methodsFor: 'omni reflection' stamp: ''!
requestExecutionOf: t1 with: t2 with: t3 with: t4 with: t5 with: t6 with: t7 with: t8 with: t9 with: t10 
	<omniDoNotTransform>
	^ omniDomain
		requestExecutionOf: t1
		on: self
		with: t2
		with: t3
		with: t4
		with: t5
		with: t6
		with: t7
		with: t8
		with: t9
		with: t10! !

!BGRepeatStream methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:40'!
requestExecutionOf: aSel with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9 with: p10
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		with: p2
		with: p3
		with: p4
		with: p5
		with: p6
		with: p7
		with: p8 with: p9 with: p10
		of: aSel
		on: self! !

!BGRepeatStream methodsFor: 'initialize-release' stamp: ''!
to: anInteger
   repeatPtr := 0.
   repeatLimit := anInteger! !
BGMall initialize!
BGChameneosColour initialize!
BGRandomNumber initialize!
